from unittest import TestCase
from .action import action_strategy
from abstra_internals.repositories.stage_run import LocalStageRunRepository
from abstra_internals.repositories.project.project import (
    FormStage,
    WorkflowTransition,
    JobStage,
)


class ActionStrategyTest(TestCase):
    def setUp(self) -> None:
        self.repository = LocalStageRunRepository()

    def test_advances_when_finished(self):
        stage = JobStage(
            id="1",
            title="title",
            file="job.py",
            schedule="* * * * *",
            workflow_position=(0, 0),
            workflow_transitions=[
                WorkflowTransition(
                    id="1",
                    target_type="target_type",
                    target_id="target_id",
                    type="jobs:finished",
                )
            ],
        )
        stage_run = self.repository.create_initial(stage="1", data={})
        result = action_strategy(stage, stage_run, "jobs:finished")
        self.assertEqual(result, [dict(stage="target_id", data={})])

    def test_stays_when_abandoned(self):
        stage = FormStage(
            id="1",
            title="title",
            file="job.py",
            path="path",
            is_initial=False,
            workflow_position=(0, 0),
            workflow_transitions=[
                WorkflowTransition(
                    id="1",
                    target_type="target_type",
                    target_id="target_id",
                    type="jobs:finished",
                )
            ],
        )
        stage_run = self.repository.create_initial(stage="1", data={})
        result = action_strategy(stage, stage_run, "forms:abandoned")
        self.assertEqual(result, [dict(stage="1", data={})])



from unittest import TestCase
from tests.utils.match import any_of
from .jwt_auth import decode_jwt, endcode_fake_jwt


class DecodeJwtTest(TestCase):
    def test_decode_jwt(self):
        self.assertEqual(
            decode_jwt(
                "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJlbWFpbCI6InRlc3RAdGVzdC5jb20ifQ.BqjOCMNr_fwqTK6VnbGp9WhE_sxR6P_cLG_GjKPoZcc"
            ),
            {"email": "test@test.com"},
        )


class EncodeFakeJwtTest(TestCase):
    def test_encode_fake_jwt(self):
        self.maxDiff = None
        self.assertEqual(
            decode_jwt(endcode_fake_jwt("test@test.com")),
            {
                "email": "test@test.com",
                "exp": any_of(int),
            },
        )



"""
This file is auto generated. Do not edit this file manually.
Source code: [monorepo]/contracts/generator.py
Last generated: 2024-01-17 20:26:56.715597
"""

import typing
from dataclasses import dataclass

CommonUserId = str

CommonUserEmail = str

CommonUserRolesItemValues = ["workflow_viewer"]

CommonUserRolesItem = typing.Literal["workflow_viewer"]

CommonUserRoles = typing.List[CommonUserRolesItem]

CommonUserProjectId = str

CommonUserCreatedAt = str


@dataclass
class CommonUser:
    id: CommonUserId
    email: CommonUserEmail
    roles: CommonUserRoles
    projectId: CommonUserProjectId
    createdAt: CommonUserCreatedAt


CloudApiCliUserGetResponse = CommonUser

CloudApiCliAuthInfoResponseAuthorId = str

CloudApiCliAuthInfoResponseProjectId = str


@dataclass
class CloudApiCliAuthInfoResponse:
    authorId: CloudApiCliAuthInfoResponseAuthorId
    projectId: CloudApiCliAuthInfoResponseProjectId


CloudApiCliBuildCreateResponseUrl = str

CloudApiCliBuildCreateResponseBuildId = str


@dataclass
class CloudApiCliBuildCreateResponse:
    url: CloudApiCliBuildCreateResponseUrl
    buildId: CloudApiCliBuildCreateResponseBuildId


AbstraLibApiStageCardContentKey = str

AbstraLibApiStageCardContentValue = typing.Dict[str, typing.Any]

AbstraLibApiStageCardContentType = str


@dataclass
class AbstraLibApiStageCardContent:
    key: AbstraLibApiStageCardContentKey
    value: AbstraLibApiStageCardContentValue
    type: AbstraLibApiStageCardContentType


AbstraLibApiStageCardContentsItem = AbstraLibApiStageCardContent

AbstraLibApiStageCardContents = typing.List[AbstraLibApiStageCardContentsItem]

AbstraLibApiVisualizationName = str

AbstraLibApiVisualizationTypeValues = ["text", "image", "table", "progress-bar"]

AbstraLibApiVisualizationType = typing.Union[
    typing.Literal["text"],
    typing.Literal["image"],
    typing.Literal["table"],
    typing.Literal["progress-bar"],
]


@dataclass
class AbstraLibApiVisualization:
    name: AbstraLibApiVisualizationName
    type: AbstraLibApiVisualizationType


AbstraLibApiStageRunId = str

AbstraLibApiStageRunStage = str

AbstraLibApiStageRunData = typing.Dict[str, typing.Any]

AbstraLibApiStageRunStatusValues = [
    "waiting",
    "running",
    "finished",
    "failed",
    "abandoned",
]

AbstraLibApiStageRunStatus = typing.Union[
    typing.Literal["waiting"],
    typing.Literal["running"],
    typing.Literal["finished"],
    typing.Literal["failed"],
    typing.Literal["abandoned"],
]

AbstraLibApiStageRunCreatedAt = str

AbstraLibApiStageRunParentId = str

AbstraLibApiStageRunExecutionId = str


@dataclass
class AbstraLibApiStageRun:
    id: AbstraLibApiStageRunId
    stage: AbstraLibApiStageRunStage
    data: AbstraLibApiStageRunData
    status: AbstraLibApiStageRunStatus
    created_at: AbstraLibApiStageRunCreatedAt
    parent_id: typing.Optional[AbstraLibApiStageRunParentId]
    execution_id: typing.Optional[AbstraLibApiStageRunExecutionId]


AbstraLibApiStageRunCardId = str

AbstraLibApiStageRunCardCreatedAt = str

AbstraLibApiStageRunCardAssignee = str

AbstraLibApiStageRunCardStatus = AbstraLibApiStageRunStatus

AbstraLibApiStageRunCardContent = AbstraLibApiStageCardContents


@dataclass
class AbstraLibApiStageRunCard:
    id: AbstraLibApiStageRunCardId
    created_at: AbstraLibApiStageRunCardCreatedAt
    assignee: typing.Optional[AbstraLibApiStageRunCardAssignee]
    status: AbstraLibApiStageRunCardStatus
    content: AbstraLibApiStageRunCardContent


AbstraLibApiStageRunCardsItem = AbstraLibApiStageRunCard

AbstraLibApiStageRunCards = typing.List[AbstraLibApiStageRunCardsItem]

AbstraLibApiStageId = str

AbstraLibApiStageTypeValues = ["form", "hook", "job", "script", "iterator", "condition"]

AbstraLibApiStageType = typing.Union[
    typing.Literal["form"],
    typing.Literal["hook"],
    typing.Literal["job"],
    typing.Literal["script"],
    typing.Literal["iterator"],
    typing.Literal["condition"],
]

AbstraLibApiStageTitle = str

AbstraLibApiStagePath = str

AbstraLibApiStageCanBeStarted = bool


@dataclass
class AbstraLibApiStage:
    id: AbstraLibApiStageId
    type: AbstraLibApiStageType
    title: AbstraLibApiStageTitle
    path: typing.Optional[AbstraLibApiStagePath]
    can_be_started: AbstraLibApiStageCanBeStarted


AbstraLibApiKanbanColumnSelectedStage = AbstraLibApiStage

AbstraLibApiKanbanColumnStageRunCards = AbstraLibApiStageRunCards

AbstraLibApiStagesItem = AbstraLibApiStage

AbstraLibApiStages = typing.List[AbstraLibApiStagesItem]

AbstraLibApiKanbanColumnStages = AbstraLibApiStages


@dataclass
class AbstraLibApiKanbanColumn:
    selected_stage: AbstraLibApiKanbanColumnSelectedStage
    stage_run_cards: AbstraLibApiKanbanColumnStageRunCards
    stages: AbstraLibApiKanbanColumnStages


AbstraLibApiKanbanColumnsItem = AbstraLibApiKanbanColumn

AbstraLibApiKanbanColumns = typing.List[AbstraLibApiKanbanColumnsItem]

AbstraLibApiKanbanDataColumns = AbstraLibApiKanbanColumns

AbstraLibApiKanbanDataNextStageOptions = AbstraLibApiStages


@dataclass
class AbstraLibApiKanbanData:
    columns: AbstraLibApiKanbanDataColumns
    next_stage_options: AbstraLibApiKanbanDataNextStageOptions


AbstraLibApiLocalLogMessageEvent = str

AbstraLibApiLocalLogMessageExecutionId = str

AbstraLibApiLocalLogMessagePayload = typing.Dict[str, str]

AbstraLibApiLocalLogMessageRuntimeType = str

AbstraLibApiLocalLogMessageRuntimeName = str

AbstraLibApiLocalLogMessageCreatedAt = float


@dataclass
class AbstraLibApiLocalLogMessage:
    event: AbstraLibApiLocalLogMessageEvent
    executionId: AbstraLibApiLocalLogMessageExecutionId
    payload: AbstraLibApiLocalLogMessagePayload
    runtimeType: AbstraLibApiLocalLogMessageRuntimeType
    runtimeName: AbstraLibApiLocalLogMessageRuntimeName
    createdAt: AbstraLibApiLocalLogMessageCreatedAt


AbstraLibApiStageRunLogsStageRun = AbstraLibApiStageRun

AbstraLibApiStageRunLogsLogsItem = AbstraLibApiLocalLogMessage

AbstraLibApiStageRunLogsLogs = typing.List[AbstraLibApiStageRunLogsLogsItem]

AbstraLibApiStageRunLogsStage = AbstraLibApiStage


@dataclass
class AbstraLibApiStageRunLogs:
    stage_run: AbstraLibApiStageRunLogsStageRun
    logs: AbstraLibApiStageRunLogsLogs
    stage: AbstraLibApiStageRunLogsStage


AbstraLibApiVisualizationsItem = AbstraLibApiVisualization

AbstraLibApiVisualizations = typing.List[AbstraLibApiVisualizationsItem]



import flask

from ...usage import usage
from .main import MainController


def get_editor_bp(controller: MainController):
    bp = flask.Blueprint("editor_linters", __name__)

    @bp.get("/check")
    def _check_linters():
        return controller.check_linters()

    @bp.post("/fix/<rule_name>/<fix_name>")
    @usage
    def _fix_linter(rule_name: str, fix_name: str):
        controller.fix_linter(rule_name, fix_name)
        return {"success": True}

    return bp



import flask, flask_cors

from ..blueprints.editor import get_editor_bp
from ..blueprints.player import get_player_bp
from ..controller import MainController


def get_local_app(controller: MainController) -> flask.Flask:
    app = flask.Flask(__name__)
    app.url_map.strict_slashes = False
    flask_cors.CORS(app)

    editor = get_editor_bp(controller)
    app.register_blueprint(editor, url_prefix="/_editor")

    player = get_player_bp(controller)
    app.register_blueprint(player)

    return app



from abstra_internals.contract import Message
from abstra_internals.execution.form_execution import get_form_execution_throwable


def send(message: Message):
    execution = get_form_execution_throwable()
    return execution.send(message)


def receive():
    execution = get_form_execution_throwable()
    return execution.receive()



import json
from ...settings import Settings
from ...templates import vscode_launch_json

CONNECTED_PORT = None


def is_launch_json_configured():
    launch_json_file = Settings.root_path.joinpath(".vscode/launch.json")
    if not launch_json_file.exists():
        return False

    launch_json = json.loads(launch_json_file.read_text(encoding="utf-8"))
    if launch_json["version"] != "0.2.0":
        return False

    configurations = launch_json["configurations"]
    if len(configurations) == 0:
        return False
    configuration = [c for c in configurations if c["name"] == "Abstra debugger"][0]
    if configuration["type"] != "python":
        return False
    if configuration["request"] != "attach":
        return False
    if configuration["connect"]["host"] not in ["localhost", "127.0.0.1", "0.0.0.0"]:
        return False
    if configuration["connect"]["port"] != "${input:abstraDebugPort}":
        return False
    inputs = launch_json["inputs"]
    if len(inputs) == 0:
        return False
    input = [i for i in inputs if i["id"] == "abstraDebugPort"][0]
    if input["type"] != "promptString":
        return False
    if input["id"] != "abstraDebugPort":
        return False
    return True


def configure_launch_json():
    if is_launch_json_configured():
        return
    Settings.root_path.joinpath(".vscode").mkdir(exist_ok=True)
    Settings.root_path.joinpath(".vscode/launch.json").write_text(
        json.dumps(vscode_launch_json, indent=4), encoding="utf-8"
    )



from ..widget_base import OptionalListInput
from typing import List, Any, Union
import json
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import pandas as pd


class PandasRowSelectionInput(OptionalListInput):
    type = "pandas-row-selection-input"
    empty_value: Union[List, Any] = None
    multiple: bool = False
    df: "pd.DataFrame"

    def __init__(self, key: str, df: Any = None, **kwargs):
        super().__init__(key)
        self.set_props(dict(df=df, **kwargs))

    def set_props(self, props):
        self.df = props.get("df", "")
        self.required = props.get("required", True)
        self.hint = props.get("hint", None)
        self.full_width = props.get("full_width", False)
        self.display_index = props.get("display_index", False)
        self.disabled = props.get("disabled", False)
        self.label = props.get("label", "")
        self.filterable = props.get("filterable", False)
        self.multiple = props.get("multiple", False)
        self.empty_value = [] if self.multiple else None
        self.value = props.get("initial_value", self.empty_value)

    def serialize_table(self):
        if self.df is None:
            import pandas as pd

            df = pd.DataFrame({"change the": [1, 2, 3], "df property": [4, 5, 6]})
            df_json = df.to_json(orient="table")
            if not isinstance(df_json, str):
                raise Exception("df.to_json() did not return a string")
            serialized = json.loads(df_json)
            del serialized["schema"]["pandas_version"]
            return serialized
        df_json = str(self.df.to_json(orient="table"))
        serialized = json.loads(df_json)
        del serialized["schema"]["pandas_version"]
        return serialized

    def render(self, ctx: dict):
        return {
            "type": self.type,
            "key": self.key,
            "hint": self.hint,
            "table": self.serialize_table(),
            "required": self.required,
            "fullWidth": self.full_width,
            "displayIndex": self.display_index,
            "disabled": self.disabled,
            "label": self.label,
            "multiple": self.multiple,
            "filterable": self.filterable,
            "value": self.serialize_value(),
            "errors": self.errors,
        }

    def serialize_value(self) -> List:
        if isinstance(self.value, list):
            return self.value
        if self.value is None:
            return []
        return [self.value]



from typing import Union, get_args, get_origin


def check_type(obj, expected_type):
    if get_origin(expected_type) is Union:
        for t in get_args(expected_type):
            if isinstance(obj, t):
                return True
        return False
    else:
        return isinstance(obj, expected_type)


def validate_json(json, expected_types):
    validated_json = {}
    for key, expected_type in expected_types.items():
        if key in json and check_type(json[key], expected_type):
            validated_json[key] = json[key]
    return validated_json



from unittest import TestCase
from pkg_resources import get_distribution
from tests.fixtures import init_dir, clear_dir
from abstra_internals.server.controller import MainController
from .missing_abstra_in_requirements import MissingAbstraInRequirements


class MissingAbstraInRequirementsTest(TestCase):
    def setUp(self) -> None:
        self.root = init_dir()

    def tearDown(self) -> None:
        clear_dir(self.root)

    def test_missing_abstra_in_requirements_valid_default(self):
        MainController()
        rule = MissingAbstraInRequirements()
        self.assertEqual(len(rule.find_issues()), 0)

    def test_missing_abstra_in_requirements_valid_with_requirements(self):
        rule = MissingAbstraInRequirements()
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.write_text(f"abstra=={get_distribution('abstra').version}")
        self.assertEqual(len(rule.find_issues()), 0)

    def test_missing_abstra_in_requirements_invalid_with_different_version(self):
        rule = MissingAbstraInRequirements()
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.write_text(f"abstra==0.0.1")
        self.assertEqual(len(rule.find_issues()), 1)

    def test_missing_abstra_in_requirements_invalid_without_requirements(self):
        rule = MissingAbstraInRequirements()
        self.assertEqual(len(rule.find_issues()), 1)

    def test_missing_abstra_in_requirements_fix(self):
        rule = MissingAbstraInRequirements()
        self.assertEqual(len(rule.find_issues()), 1)
        rule.find_issues()[0].fixes[0].fix()
        self.assertEqual(len(rule.find_issues()), 0)
        requirements_txt = self.root / "requirements.txt"
        self.assertTrue(requirements_txt.exists())
        with requirements_txt.open("r") as file:
            content = file.read()
            self.assertTrue(f"abstra=={get_distribution('abstra').version}" in content)



import sys
import traceback


def format_exception(exception, limit=None, chain=True):
    if sys.version_info >= (3, 10):
        # For Python 3.10 and later
        # https://docs.python.org/3/library/traceback.html#traceback.format_exception

        return traceback.format_exception(exception, limit=limit, chain=chain)
    else:
        # For Python 3.9 and earlier
        return traceback.format_exception(
            type(exception), exception, exception.__traceback__, limit=limit
        )



import requests, typing, dataclasses

from .threaded import threaded
from .utils.environment import SIDECAR_URL, SIDECAR_HEADERS


@dataclasses.dataclass
class LogMessage:
    event: str
    executionId: str
    runtime_type: str
    runtime_name: str
    payload: typing.Dict[str, str]

    def json(self):
        return dict(
            event=str(self.event),
            payload=dict(self.payload),
            executionId=str(self.executionId),
            runtimeType=str(self.runtime_type),
            runtimeName=str(self.runtime_name),
        )


@threaded
def log(msg: LogMessage):
    if not SIDECAR_URL:
        return
    try:
        requests.post(f"{SIDECAR_URL}/log", headers=SIDECAR_HEADERS, json=msg.json())
    except Exception:
        pass  # TODO: handle this
